/*
 * CPM (clock and power management) support for the JZ4780.
 *
 * Copyright (C) 2017, 2018, 2020, 2021, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/devices/clocks.h>
#include <l4/sys/types.h>
#include <stdint.h>



#ifdef __cplusplus

#include <l4/devices/cpm.h>

class Cpm_jz4780_chip : public Cpm_chip
{
public:
  explicit Cpm_jz4780_chip(l4_addr_t addr);
};

#endif /* __cplusplus */



/* C language interface. */

EXTERN_C_BEGIN

void *jz4780_cpm_init(l4_addr_t cpm_base);

const char *jz4780_cpm_clock_type(void *cpm, enum Clock_identifiers clock);

int jz4780_cpm_have_clock(void *cpm, enum Clock_identifiers clock);
void jz4780_cpm_start_clock(void *cpm, enum Clock_identifiers clock);
void jz4780_cpm_stop_clock(void *cpm, enum Clock_identifiers clock);

int jz4780_cpm_get_parameters(void *cpm, enum Clock_identifiers clock,
                              uint32_t parameters[]);
int jz4780_cpm_set_parameters(void *cpm, enum Clock_identifiers clock,
                              int num_parameters, uint32_t parameters[]);

uint8_t jz4780_cpm_get_source(void *cpm, enum Clock_identifiers clock);
void jz4780_cpm_set_source(void *cpm, enum Clock_identifiers clock, uint8_t source);
enum Clock_identifiers jz4780_cpm_get_source_clock(void *cpm, enum Clock_identifiers clock);
void jz4780_cpm_set_source_clock(void *cpm, enum Clock_identifiers clock, enum Clock_identifiers source);

uint32_t jz4780_cpm_get_source_frequency(void *cpm, enum Clock_identifiers clock);

uint32_t jz4780_cpm_get_frequency(void *cpm, enum Clock_identifiers clock);
int jz4780_cpm_set_frequency(void *cpm, enum Clock_identifiers clock, uint32_t frequency);

EXTERN_C_END
